HUGGING_FACE = "huggingface"
EOS_BUCKET = "eos-bucket"


LLAMA2 = "meta-llama/Llama-2-7b-hf"
MISTRAL_7B = "mistralai/Mistral-7B-v0.1"
STABLE_DIFFUSION_2_1 = "stabilityai/stable-diffusion-2-1"
MISTRAL_8X7B = "mistralai/Mixtral-8x7B-v0.1"
GEMMA_7B = "google/gemma-7b"
LLAMA3_8B = "meta-llama/Meta-Llama-3-8B"
TEXT_MODELS_LIST = [LLAMA2, LLAMA3_8B, MISTRAL_7B, MISTRAL_8X7B, GEMMA_7B]
